require 'active_model'
require 'base64'
Dir["#{File.dirname(__FILE__)}/*_validator.rb"].each {|f| require f}
require 'vas_exceptions'
require 'xml_addin'
require 'conn_addin'

# Provides a base from which to build objects that utilize the OpenvasCli object.
# Includes support for ActiveModel validations and a helper method (extract_value_from)
# to assist in OpenVAS object implementations.
module OpenvasCli
  class VasBase
    include ActiveModel::Validations
    include ActiveModel::Dirty
    include XmlAddin
    include ConnAddin
    
    has_vas_connection(:manager)
    
    attr_accessor :id
    
    def initialize(attributes={})
      attributes.each { |key, value|
        send("#{key}=".to_sym, value) if public_methods.include?("#{key}=")
      }
      reset_changes
    end
    
    def update_attributes(attrs={})
      attrs.each { |key, value|
        send("#{key}=".to_sym, value) if public_methods.include?("#{key}=")
      }
    end
    
    def new_record?
      @id == nil || @id.empty?
    end
    
    def reset_changes
      @changed_attributes.clear if @changed_attributes 
    end
    
    def to_param
      id
    end
    
    def to_key
      if new_record?
        nil
      else
        [@id]
      end
    end
    
    def save!
      raise(VasExceptions::RecordNotSaved) unless valid?
      create_or_update || raise(VasExceptions::RecordNotSaved)
    end
    
    def save
      create_or_update if valid?      
    end
    
    def destroy!
      delete_record || raise(VasExceptions::RecordNotSaved)
    end
    
    def destroy
      delete_record
    end
    
    def self.get_by_id(id)
      get_all(:id => id).first
    end
    
    def create_or_update
      true
    end
    
    def delete_record
      true
    end
  end
end
