require 'vas_base'

module OpenvasCli
  # Describes a NSIS/OpenVAS scan rule.
  #--
  # TODO: Filter by other useful fields
  # TODO: Find external links for more information
  # TODO: Redcloth and Autolink description
  # TODO: Implement sorting options[:sort_by]
  #++
  class VasNVT < VasBase
    # Human readable name of the rule.
    attr_accessor :name
    
    attr_accessor :category
    attr_accessor :copyright
    attr_accessor :description
    attr_accessor :summary 
    attr_accessor :family
    attr_accessor :version
    attr_accessor :cvss_base
    attr_accessor :risk_factor
    attr_accessor :cve_id
    attr_accessor :bugtraq_id
    attr_accessor :xrefs
    attr_accessor :tags
    attr_accessor :preferences
    
    validates :id, :OID => true, :presence => true
    # Pulls VasNVT rules that match the provided options.
    #
    # === Options:
    # [:id => [VasNVT.oid]] will pull only the NVT rule that matches the given OID
    # [:family => [VasFamily.name]] will pull all NVT rules for the specified family
    def self.get_all(options = {})
      params = {:details => '1'}
      params[:nvt_oid] = options[:id] if options[:id]
      params[:family]  = options[:family] if options[:family]
      params[:config_id] = options[:config_id] if options[:config_id]
    
      req = Nokogiri::XML::Builder.new { |xml|
        xml.get_nvts(params)
      }
      
      rules = connection.send_receive(req.doc)
      
      ret = []
      rules.xpath("/get_nvts_response/nvt").each { |nvt|
        rule             = VasNVT.new
        rule.id          = extract_value_from("@oid", nvt)
        rule.name        = extract_value_from("name", nvt)
        rule.category    = extract_value_from("category", nvt)
        rule.copyright   = extract_value_from("copyright", nvt)
        rule.description = extract_value_from("description", nvt).strip
        rule.summary     = extract_value_from("summary", nvt)
        rule.family      = extract_value_from("family", nvt)
        rule.version     = extract_value_from("version", nvt)
        rule.cvss_base   = extract_value_from("cvss_base", nvt)
        rule.risk_factor = extract_value_from("risk_factor", nvt)
        rule.cve_id      = extract_value_from("cve_id", nvt)
        rule.bugtraq_id  = extract_value_from("bugtraq_id", nvt)
        rule.xrefs       = extract_value_from("xrefs", nvt).split(/, ?/)
        rule.tags        = extract_value_from("tags", nvt).split(/, ?/)
        ret << rule
      }
      
      ret
    end  
  end
end
